@@ -13,9 +13,7 @@ use Carp;
 
 use Tangence::Constants;
 
-require Tangence::Type;
-
-our $VERSION = '0.20_001';
+our $VERSION = '0.16';
 
 sub build_accessor
 {
@@ -210,12 +208,6 @@ sub _accessor_for_objset
    };
 }
 
-sub make_type
-{
-   shift;
-   return Tangence::Type->new( @_ );
-}
-
 package # hide from CPAN
    Tangence::Property::_Iterator;
 
