@@ -9,13 +9,23 @@ use Tangence::Constants;
 sub new
 {
    my $class = shift;
+   my %args = @_;
+   my $self = $class->SUPER::new( %args );
 
-   my $self = $class->SUPER::new( @_ );
-
+   for (qw( scalar array queue hash s_scalar )) {
+      $self->${\"set_prop_$_"}( $args{$_} ) if defined $args{$_};
+   }
 
    return $self;
 }
 
+sub method_method
+{
+   my $self = shift;
+   my ( $ctx, $i, $s ) = @_;
+   return "$i/$s";
+}
+
 sub init_prop_scalar { 123 }
 sub init_prop_hash   { { one => 1, two => 2, three => 3 } }
 sub init_prop_queue  { [ 1, 2, 3 ] }
